@@ -1,9 +1,11 @@
-from parsl.providers import AdHocProvider
-from parsl.channels import SSHChannel
-from parsl.executors import HighThroughputExecutor
-from parsl.config import Config
 from typing import Any, Dict
 
+from parsl.channels import SSHChannel
+from parsl.config import Config
+from parsl.executors import HighThroughputExecutor
+from parsl.providers import AdHocProvider
+from parsl.usage_tracking.levels import LEVEL_1
+
 user_opts: Dict[str, Dict[str, Any]]
 user_opts = {'adhoc':
              {'username': 'YOUR_USERNAME',
@@ -32,4 +34,5 @@ config = Config(
     ],
     #  AdHoc Clusters should not be setup with scaling strategy.
     strategy='none',
+    usage_tracking=LEVEL_1,
 )
