@@ -1,8 +1,8 @@
-from parsl.providers import LocalProvider
 from parsl.channels import LocalChannel
-
 from parsl.config import Config
 from parsl.executors import HighThroughputExecutor
+from parsl.providers import LocalProvider
+from parsl.usage_tracking.levels import LEVEL_1
 
 config = Config(
     executors=[
@@ -16,4 +16,5 @@ config = Config(
             ),
         )
     ],
+    usage_tracking=LEVEL_1,
 )
