@@ -1,16 +1,20 @@
+import datetime
+import logging
 import os
 import time
-import logging
-import datetime
 from functools import wraps
-
-from parsl.multiprocessing import ForkProcess
 from multiprocessing import Event
-from parsl.process_loggers import wrap_with_logs
+from typing import Any, Callable, Dict, List, Sequence, Tuple
 
 from parsl.monitoring.message_type import MessageType
-from parsl.monitoring.radios import MonitoringRadio, UDPRadio, HTEXRadio, FilesystemRadio
-from typing import Any, Callable, Dict, List, Sequence, Tuple
+from parsl.monitoring.radios import (
+    FilesystemRadioSender,
+    HTEXRadioSender,
+    MonitoringRadioSender,
+    UDPRadioSender,
+)
+from parsl.multiprocessing import ForkProcess
+from parsl.process_loggers import wrap_with_logs
 
 logger = logging.getLogger(__name__)
 
@@ -96,17 +100,17 @@ def monitor_wrapper(*,
     return (wrapped, args, new_kwargs)
 
 
-def get_radio(radio_mode: str, monitoring_hub_url: str, task_id: int, run_dir: str) -> MonitoringRadio:
-    radio: MonitoringRadio
+def get_radio(radio_mode: str, monitoring_hub_url: str, task_id: int, run_dir: str) -> MonitoringRadioSender:
+    radio: MonitoringRadioSender
     if radio_mode == "udp":
-        radio = UDPRadio(monitoring_hub_url,
-                         source_id=task_id)
+        radio = UDPRadioSender(monitoring_hub_url,
+                               source_id=task_id)
     elif radio_mode == "htex":
-        radio = HTEXRadio(monitoring_hub_url,
-                          source_id=task_id)
+        radio = HTEXRadioSender(monitoring_hub_url,
+                                source_id=task_id)
     elif radio_mode == "filesystem":
-        radio = FilesystemRadio(monitoring_url=monitoring_hub_url,
-                                source_id=task_id, run_dir=run_dir)
+        radio = FilesystemRadioSender(monitoring_url=monitoring_hub_url,
+                                      source_id=task_id, run_dir=run_dir)
     else:
         raise RuntimeError(f"Unknown radio mode: {radio_mode}")
     return radio
@@ -135,8 +139,8 @@ def send_first_last_message(try_id: int,
                             monitoring_hub_url: str,
                             run_id: str, radio_mode: str, run_dir: str,
                             is_last: bool) -> None:
-    import platform
     import os
+    import platform
 
     radio = get_radio(radio_mode, monitoring_hub_url, task_id, run_dir)
 
@@ -177,6 +181,7 @@ def monitor(pid: int,
     """
     import logging
     import platform
+
     import psutil
 
     from parsl.utils import setproctitle
